module Model
  class Capability
    attr_reader :query

    def initialize(node, repository)
      @query = NodeQuery.new(node, repository)
    end

    def description
      query.value(LV.description)
    end

    def value
      query.value(LV.value).to_s if query.value_exists?(LV.value)
    end

    def type
      value = query.value(RDF.type)
      return nil if value.nil?
      t = value.to_s.remove LV.prefix
      Helper.underscore(t).to_sym
    end

    def device
      device_node = query.value(LV.device)
      Device.new device_node, query.repository if device_node
    end

    def property
      node = query.value(LV.property)
      Property.new node, query.repository if node
    end

    def question
      node = query.value(LV.question)
      Question.new node, query.repository if node
    end

    def actuators
      query.values(LV.actuator).map do |node|
        Device.new node, query.repository
      end
    end

    def sensors
      query.values(LV.sensor).map do |node|
        Device.new node, query.repository
      end
    end
  end
end
