module Model
  class Installation
    attr_reader :query

    def initialize(node, repository)
      @query = NodeQuery.new(node, repository)
    end

    def type
      query.value(LV.type).to_s
    end

    def name
      query.value(LV.name)
    end

    def description
      query.value(LV.description)
    end

    def category
      value = query.value(LV.category)
      return nil if value.nil?
      t = value.to_s.remove LV.prefix
      Helper.underscore(t).to_sym
    end

    def used_devices
      query.values(LV.uses).map do |node|
        InstallationUsedDevice.new(node, query.repository)
      end
    end

    def questions
      query.values(LV.question).map do |node|
        Question.new(node, query.repository)
      end
    end

    def properties
      query.values(LV.property).map do |node|
        Property.new(node, query.repository)
      end
    end
  end
end
